@@ -84,7 +84,8 @@ typedef struct {
 } GstPluginDep;
 
 struct _GstPluginPrivate {
-  GList *deps;    /* list of GstPluginDep structures */
+  GList *deps;                 /* list of GstPluginDep structures */
+  GstStructure *status_info;
   GstStructure *cache_data;
 };
 
@@ -154,7 +155,7 @@ gboolean _priv_gst_registry_remove_cache_plugins (GstRegistry *registry);
 G_GNUC_INTERNAL  void _priv_gst_registry_cleanup (void);
 
 GST_API
-gboolean _gst_plugin_loader_client_run (void);
+gboolean _gst_plugin_loader_client_run (const gchar * pipe_name);
 
 G_GNUC_INTERNAL  GstPlugin * _priv_gst_plugin_load_file_for_registry (const gchar *filename,
                                                                       GstRegistry * registry,
@@ -167,7 +168,7 @@ G_GNUC_INTERNAL const char * _priv_gst_value_gtype_to_abbr (GType type);
 G_GNUC_INTERNAL gboolean _priv_gst_value_parse_string (gchar * s, gchar ** end, gchar ** next, gboolean unescape);
 G_GNUC_INTERNAL gboolean _priv_gst_value_parse_simple_string (gchar * str, gchar ** end);
 G_GNUC_INTERNAL gboolean _priv_gst_value_parse_value (gchar * str, gchar ** after, GValue * value, GType default_type, GParamSpec *pspec);
-G_GNUC_INTERNAL gchar * _priv_gst_value_serialize_any_list (const GValue * value, const gchar * begin, const gchar * end, gboolean print_type);
+G_GNUC_INTERNAL gchar * _priv_gst_value_serialize_any_list (const GValue * value, const gchar * begin, const gchar * end, gboolean print_type, GstSerializeFlags flags);
 
 /* Used in GstBin for manual state handling */
 G_GNUC_INTERNAL  void _priv_gst_element_state_changed (GstElement *element,
@@ -348,12 +349,6 @@ extern GstClockTime _priv_gst_start_time;
 
 #endif
 
-#ifdef GST_DISABLE_GST_DEBUG
-/* for _gst_element_error_printf */
-#define __gst_vasprintf __gst_info_fallback_vasprintf
-int __gst_vasprintf (char **result, char const *format, va_list args);
-#endif
-
 /**** objects made opaque until the private bits have been made private ****/
 
 #include <gmodule.h>
@@ -514,14 +509,11 @@ struct _GstDynamicTypeFactoryClass {
 /* privat flag used by GstBus / GstMessage */
 #define GST_MESSAGE_FLAG_ASYNC_DELIVERY (GST_MINI_OBJECT_FLAG_LAST << 0)
 
-/* private type used by GstClock */
-typedef struct _GstClockWeakRef GstClockWeakRef;
-
 /* private struct used by GstClock and GstSystemClock */
 struct _GstClockEntryImpl
 {
   GstClockEntry entry;
-  GstClockWeakRef *weakref;
+  GWeakRef *clock;
   GDestroyNotify destroy_entry;
   gpointer padding[21];                 /* padding for allowing e.g. systemclock
                                          * to add data in lieu of overridable
@@ -529,6 +521,11 @@ struct _GstClockEntryImpl
 };
 
 char * priv_gst_get_relocated_libgstreamer (void);
+gint   priv_gst_count_directories (const char *filepath);
+
+void priv_gst_clock_init (void);
+GstClockTime priv_gst_get_monotonic_time (void);
+GstClockTime priv_gst_get_real_time (void);
 
 G_END_DECLS
 #endif /* __GST_PRIVATE_H__ */
