@@ -222,7 +222,7 @@ _gst_caps_free (GstCaps * caps)
   memset (caps, 0xff, sizeof (GstCapsImpl));
 #endif
 
-  g_slice_free1 (sizeof (GstCapsImpl), caps);
+  g_free (caps);
 }
 
 static void
@@ -255,7 +255,7 @@ gst_caps_new_empty (void)
 {
   GstCaps *caps;
 
-  caps = (GstCaps *) g_slice_new (GstCapsImpl);
+  caps = (GstCaps *) g_new (GstCapsImpl, 1);
 
   gst_caps_init (caps);
 
@@ -2646,10 +2646,9 @@ gst_caps_map_in_place (GstCaps * caps, GstCapsMapFunc func, gpointer user_data)
  *
  * Calls the provided function once for each structure and caps feature in the
  * #GstCaps. In contrast to gst_caps_foreach(), the function may modify the
- * structure and features. In contrast to gst_caps_filter_and_map_in_place(),
- * the structure and features are removed from the caps if %FALSE is returned
- * from the function.
- * The caps must be mutable.
+ * structure and features. In contrast to gst_caps_map_in_place(), the structure
+ * and features are removed from the caps if %FALSE is returned from the
+ * function. The caps must be mutable.
  *
  * Since: 1.6
  */
