@@ -165,11 +165,13 @@ struct _GstClockPrivate
   gint post_count;
 
   gboolean synced;
+
+  GWeakRef *clock_weakref;
 };
 
 typedef struct _GstClockEntryImpl GstClockEntryImpl;
 
-#define GST_CLOCK_ENTRY_CLOCK_WEAK_REF(entry) (&((GstClockEntryImpl *)(entry))->clock)
+#define GST_CLOCK_ENTRY_CLOCK_WEAK_REF(entry) (((GstClockEntryImpl *)(entry))->clock)
 
 /* seqlocks */
 #define read_seqbegin(clock)                                   \
@@ -245,7 +247,7 @@ gst_clock_entry_new (GstClock * clock, GstClockTime time,
 {
   GstClockEntry *entry;
 
-  entry = (GstClockEntry *) g_slice_new0 (GstClockEntryImpl);
+  entry = (GstClockEntry *) g_new0 (GstClockEntryImpl, 1);
 
   /* FIXME: add tracer hook for struct allocations such as clock entries */
 
@@ -260,7 +262,8 @@ gst_clock_entry_new (GstClock * clock, GstClockTime time,
   entry->_clock = clock;
 #endif
 #endif
-  g_weak_ref_init (GST_CLOCK_ENTRY_CLOCK_WEAK_REF (entry), clock);
+  GST_CLOCK_ENTRY_CLOCK_WEAK_REF (entry) =
+      g_atomic_rc_box_acquire (clock->priv->clock_weakref);
   entry->type = type;
   entry->time = time;
   entry->interval = interval;
@@ -374,11 +377,12 @@ _gst_clock_id_free (GstClockID id)
   if (entry_impl->destroy_entry)
     entry_impl->destroy_entry (entry_impl);
 
-  g_weak_ref_clear (GST_CLOCK_ENTRY_CLOCK_WEAK_REF (entry));
+  g_atomic_rc_box_release_full (GST_CLOCK_ENTRY_CLOCK_WEAK_REF (entry),
+      (GDestroyNotify) g_weak_ref_clear);
 
   /* FIXME: add tracer hook for struct allocations such as clock entries */
 
-  g_slice_free (GstClockEntryImpl, (GstClockEntryImpl *) id);
+  g_free (id);
 }
 
 /**
@@ -769,6 +773,17 @@ gst_clock_init (GstClock * clock)
   priv->timeout = DEFAULT_TIMEOUT;
   priv->times = g_new0 (GstClockTime, 4 * priv->window_size);
   priv->times_temp = priv->times + 2 * priv->window_size;
+  /*
+   * An atomically ref-counted wrapper around a GWeakRef for this GstClock,
+   * created by the clock and shared with all its clock entries.
+   *
+   * This exists because g_weak_ref_ operations are quite expensive and operate
+   * with a global GRWLock. _get takes a reader lock, _init and _clear take
+   * a writer lock. We want to avoid having to instantiate a new GWeakRef for
+   * every clock entry.
+   */
+  priv->clock_weakref = g_atomic_rc_box_new (GWeakRef);
+  g_weak_ref_init (priv->clock_weakref, clock);
 }
 
 static void
@@ -801,6 +816,8 @@ gst_clock_finalize (GObject * object)
   clock->priv->times_temp = NULL;
   GST_CLOCK_SLAVE_UNLOCK (clock);
 
+  g_atomic_rc_box_release_full (clock->priv->clock_weakref,
+      (GDestroyNotify) g_weak_ref_clear);
   g_mutex_clear (&clock->priv->slave_lock);
   g_cond_clear (&clock->priv->sync_cond);
 
@@ -1741,11 +1758,11 @@ gst_clock_set_synced (GstClock * clock, gboolean synced)
           GST_CLOCK_FLAG_NEEDS_STARTUP_SYNC));
 
   GST_OBJECT_LOCK (clock);
-  if (clock->priv->synced != ! !synced) {
-    clock->priv->synced = ! !synced;
+  if (clock->priv->synced != !!synced) {
+    clock->priv->synced = !!synced;
     g_cond_signal (&clock->priv->sync_cond);
     GST_OBJECT_UNLOCK (clock);
-    g_signal_emit (clock, gst_clock_signals[SIGNAL_SYNCED], 0, ! !synced);
+    g_signal_emit (clock, gst_clock_signals[SIGNAL_SYNCED], 0, !!synced);
   } else {
     GST_OBJECT_UNLOCK (clock);
   }
