@@ -151,7 +151,7 @@ struct DeviceFilter
 static struct DeviceFilter *
 device_filter_copy (struct DeviceFilter *filter)
 {
-  struct DeviceFilter *copy = g_slice_new0 (struct DeviceFilter);
+  struct DeviceFilter *copy = g_new0 (struct DeviceFilter, 1);
 
   copy->classesv = g_strdupv (filter->classesv);
   copy->caps = filter->caps ? gst_caps_ref (filter->caps) : NULL;
@@ -167,7 +167,7 @@ device_filter_free (struct DeviceFilter *filter)
   if (filter->caps)
     gst_caps_unref (filter->caps);
 
-  g_slice_free (struct DeviceFilter, filter);
+  g_free (filter);
 }
 
 static void
@@ -257,7 +257,7 @@ update_hidden_providers_list (GList ** hidden, GstDeviceProvider * provider)
   }
 }
 
-static void
+static GstBusSyncReply
 bus_sync_message (GstBus * bus, GstMessage * message,
     GstDeviceMonitor * monitor)
 {
@@ -305,6 +305,10 @@ bus_sync_message (GstBus * bus, GstMessage * message,
     if (matches)
       gst_bus_post (monitor->priv->bus, gst_message_ref (message));
   }
+
+  gst_message_unref (message);
+
+  return GST_BUS_DROP;
 }
 
 
@@ -335,7 +339,7 @@ gst_device_monitor_remove_provider (GstDeviceMonitor * self, guint i)
   g_ptr_array_remove_index (self->priv->providers, i);
 
   bus = gst_device_provider_get_bus (provider);
-  g_signal_handlers_disconnect_by_func (bus, bus_sync_message, self);
+  gst_bus_set_sync_handler (bus, NULL, NULL, NULL);
   gst_object_unref (bus);
 
   g_signal_handlers_disconnect_by_func (provider, provider_hidden, self);
@@ -373,18 +377,6 @@ gst_device_monitor_dispose (GObject * object)
   G_OBJECT_CLASS (gst_device_monitor_parent_class)->dispose (object);
 }
 
-#if !GLIB_CHECK_VERSION(2, 60, 0)
-#define g_queue_clear_full queue_clear_full
-static void
-queue_clear_full (GQueue * queue, GDestroyNotify free_func)
-{
-  gpointer data;
-
-  while ((data = g_queue_pop_head (queue)) != NULL)
-    free_func (data);
-}
-#endif
-
 /**
  * gst_device_monitor_get_devices:
  * @monitor: A #GstDeviceProvider
@@ -673,7 +665,7 @@ gst_device_monitor_add_filter_unlocked (GstDeviceMonitor * monitor,
   guint id = 0;
   gboolean matched = FALSE;
 
-  filter = g_slice_new0 (struct DeviceFilter);
+  filter = g_new0 (struct DeviceFilter, 1);
   filter->id = monitor->priv->last_id++;
   if (caps)
     filter->caps = gst_caps_ref (caps);
@@ -715,9 +707,8 @@ gst_device_monitor_add_filter_unlocked (GstDeviceMonitor * monitor,
             (GCallback) provider_unhidden, monitor);
 
         matched = TRUE;
-        gst_bus_enable_sync_message_emission (bus);
-        g_signal_connect (bus, "sync-message",
-            G_CALLBACK (bus_sync_message), monitor);
+        gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_message,
+            monitor, NULL);
         gst_object_unref (bus);
         g_ptr_array_add (monitor->priv->providers, provider);
       }
