@@ -40,8 +40,12 @@
  * methods like gst_bin_add() and gst_bin_remove() (see #GstBin).
  *
  * Before changing the state of the #GstPipeline (see #GstElement) a #GstBus
- * can be retrieved with gst_pipeline_get_bus(). This bus can then be
- * used to receive #GstMessage from the elements in the pipeline.
+ * should be retrieved with gst_pipeline_get_bus(). This #GstBus should then
+ * be used to receive #GstMessage from the elements in the pipeline. Listening
+ * to the #GstBus is necessary for retrieving error messages from the
+ * #GstPipeline and otherwise the #GstPipeline might stop without any
+ * indication, why. Furthermore, the #GstPipeline posts messages even if
+ * nobody listens on the #GstBus, which will pile up and use up memory.
  *
  * By default, a #GstPipeline will automatically flush the pending #GstBus
  * messages when going to the NULL state to ensure that no circular
@@ -76,7 +80,7 @@
 
 #include "gst_private.h"
 #include "gsterror.h"
-#include "gst-i18n-lib.h"
+#include <glib/gi18n-lib.h>
 
 #include "gstpipeline.h"
 #include "gstinfo.h"
@@ -109,6 +113,8 @@ struct _GstPipelinePrivate
 {
   /* with LOCK */
   gboolean auto_flush_bus;
+  gboolean is_live;
+  GstClockTime min_latency;
 
   /* when we need to update stream_time or clock when going back to
    * PLAYING*/
@@ -116,6 +122,12 @@ struct _GstPipelinePrivate
   gboolean update_clock;
 
   GstClockTime latency;
+
+  /* seqnum of the most recent instant-rate-request, %GST_SEQNUM_INVALID if none */
+  guint32 instant_rate_seqnum;
+  gdouble active_instant_rate;
+  GstClockTime instant_rate_upstream_anchor;
+  GstClockTime instant_rate_clock_anchor;
 };
 
 
@@ -131,6 +143,8 @@ static GstStateChangeReturn gst_pipeline_change_state (GstElement * element,
 
 static void gst_pipeline_handle_message (GstBin * bin, GstMessage * message);
 static gboolean gst_pipeline_do_latency (GstBin * bin);
+static gboolean gst_pipeline_handle_instant_rate (GstPipeline * pipeline,
+    gdouble rate, guint32 seqnum);
 
 /* static guint gst_pipeline_signals[LAST_SIGNAL] = { 0 }; */
 
@@ -220,6 +234,9 @@ gst_pipeline_init (GstPipeline * pipeline)
   pipeline->delay = DEFAULT_DELAY;
   pipeline->priv->latency = DEFAULT_LATENCY;
 
+  pipeline->priv->is_live = FALSE;
+  pipeline->priv->min_latency = GST_CLOCK_TIME_NONE;
+
   /* create and set a default bus */
   bus = gst_bus_new ();
 #if 0
@@ -302,6 +319,15 @@ reset_start_time (GstPipeline * pipeline, GstClockTime start_time)
     GST_DEBUG_OBJECT (pipeline, "reset start_time to 0");
     GST_ELEMENT_START_TIME (pipeline) = start_time;
     pipeline->priv->last_start_time = -1;
+
+    /* Reset instant rate multiplier because we flushed / reset time.
+     * Old anchor's don't make sense */
+    pipeline->priv->instant_rate_seqnum = GST_SEQNUM_INVALID;
+    pipeline->priv->instant_rate_upstream_anchor =
+        pipeline->priv->instant_rate_clock_anchor = GST_CLOCK_TIME_NONE;
+    pipeline->priv->active_instant_rate = 1.0;
+    GST_DEBUG_OBJECT (pipeline, "Reset start time to %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (start_time));
   } else {
     GST_DEBUG_OBJECT (pipeline, "application asked to not reset stream_time");
   }
@@ -310,7 +336,7 @@ reset_start_time (GstPipeline * pipeline, GstClockTime start_time)
 
 /**
  * gst_pipeline_new:
- * @name: (allow-none): name of new pipeline
+ * @name: (nullable): name of new pipeline
  *
  * Create a new pipeline with the given name.
  *
@@ -495,6 +521,10 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
       break;
     }
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      GST_OBJECT_LOCK (element);
+      pipeline->priv->is_live = FALSE;
+      pipeline->priv->min_latency = GST_CLOCK_TIME_NONE;
+      GST_OBJECT_UNLOCK (element);
       reset_start_time (pipeline, 0);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
@@ -503,6 +533,14 @@ gst_pipeline_change_state (GstElement * element, GstStateChange transition)
 
   result = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
+  if (GST_STATE_TRANSITION_NEXT (transition) == GST_STATE_PAUSED) {
+    GST_OBJECT_LOCK (element);
+    pipeline->priv->is_live = result == GST_STATE_CHANGE_NO_PREROLL;
+    GST_INFO_OBJECT (pipeline, "pipeline is%slive",
+        pipeline->priv->is_live ? " " : " not ");
+    GST_OBJECT_UNLOCK (element);
+  }
+
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_NULL:
       break;
@@ -575,6 +613,12 @@ invalid_clock:
  * the PAUSED, pending PAUSED state. When the ASYNC_DONE message is received the
  * pipeline will redistribute the new base_time and will bring the elements back
  * to the desired state of the pipeline. */
+/* GST_MESSAGE_INSTANT_RATE_REQUEST: This message is only posted by sinks and
+ *     bins containing sinks (which are also considered sinks). Once all sinks
+ *     have posted this message it is posted to the parent bin, or if this is
+ *     a top-level bin (e.g. pipeline), a instant-rate-sync-time event with
+ *     the current running time is sent to the whole pipeline.
+ */
 static void
 gst_pipeline_handle_message (GstBin * bin, GstMessage * message)
 {
@@ -584,12 +628,24 @@ gst_pipeline_handle_message (GstBin * bin, GstMessage * message)
     case GST_MESSAGE_RESET_TIME:
     {
       GstClockTime running_time;
+      gboolean is_live;
 
       gst_message_parse_reset_time (message, &running_time);
 
       /* reset our running time if we need to distribute a new base_time to the
        * children. */
       reset_start_time (pipeline, running_time);
+
+      GST_OBJECT_LOCK (pipeline);
+      is_live = pipeline->priv->is_live;
+      GST_OBJECT_UNLOCK (pipeline);
+
+      /* If we are live, sample a new base_time immediately */
+      if (is_live && GST_STATE_TARGET (pipeline) == GST_STATE_PLAYING) {
+        gst_pipeline_change_state (GST_ELEMENT (pipeline),
+            GST_STATE_CHANGE_PAUSED_TO_PLAYING);
+      }
+
       break;
     }
     case GST_MESSAGE_CLOCK_LOST:
@@ -605,6 +661,18 @@ gst_pipeline_handle_message (GstBin * bin, GstMessage * message)
         pipeline->priv->update_clock = TRUE;
       }
       GST_OBJECT_UNLOCK (bin);
+    }
+      break;
+
+    case GST_MESSAGE_INSTANT_RATE_REQUEST:{
+      guint32 seqnum = gst_message_get_seqnum (message);
+      gdouble rate_multiplier;
+
+      gst_message_parse_instant_rate_request (message, &rate_multiplier);
+
+      gst_pipeline_handle_instant_rate (pipeline, rate_multiplier, seqnum);
+
+      break;
     }
     default:
       break;
@@ -636,6 +704,10 @@ gst_pipeline_do_latency (GstBin * bin)
 
     gst_query_parse_latency (query, &live, &min_latency, &max_latency);
 
+    GST_OBJECT_LOCK (pipeline);
+    pipeline->priv->min_latency = min_latency;
+    GST_OBJECT_UNLOCK (pipeline);
+
     GST_DEBUG_OBJECT (pipeline,
         "got min latency %" GST_TIME_FORMAT ", max latency %"
         GST_TIME_FORMAT ", live %d", GST_TIME_ARGS (min_latency),
@@ -810,7 +882,7 @@ gst_pipeline_use_clock (GstPipeline * pipeline, GstClock * clock)
 /**
  * gst_pipeline_set_clock: (skip)
  * @pipeline: a #GstPipeline
- * @clock: (transfer none): the clock to set
+ * @clock: (transfer none) (nullable): the clock to set
  *
  * Set the clock for @pipeline. The clock will be distributed
  * to all the elements managed by the pipeline.
@@ -1026,3 +1098,137 @@ gst_pipeline_get_latency (GstPipeline * pipeline)
 
   return latency;
 }
+
+static gboolean
+gst_pipeline_handle_instant_rate (GstPipeline * pipeline, gdouble rate,
+    guint32 seqnum)
+{
+  GstClockTime running_time = GST_CLOCK_TIME_NONE;
+  GstClockTime upstream_running_time = GST_CLOCK_TIME_NONE;
+  gboolean is_playing;
+  GstEvent *event;
+
+  GST_OBJECT_LOCK (pipeline);
+
+  if (pipeline->priv->instant_rate_seqnum != GST_SEQNUM_INVALID &&
+      pipeline->priv->instant_rate_seqnum == seqnum) {
+    GST_DEBUG_OBJECT (pipeline,
+        "Handling duplicate instant-rate-request message with seqnum %u",
+        seqnum);
+    upstream_running_time = pipeline->priv->instant_rate_upstream_anchor;
+    running_time = pipeline->priv->instant_rate_clock_anchor;
+
+    if (G_UNLIKELY (rate != pipeline->priv->active_instant_rate)) {
+      GST_WARNING_OBJECT (pipeline,
+          "Repeated instant-rate-request has a different rate to before! %f != %f",
+          rate, pipeline->priv->active_instant_rate);
+      rate = pipeline->priv->active_instant_rate;
+    }
+  } else {
+    /* Get the current running time of the pipeline */
+    is_playing = GST_STATE (pipeline) == GST_STATE_PLAYING
+        && (GST_STATE_PENDING (pipeline) == GST_STATE_VOID_PENDING ||
+        GST_STATE_PENDING (pipeline) == GST_STATE_PLAYING);
+
+    if (is_playing) {
+      GstClockTime base_time, clock_time;
+      GstClock *clock;
+
+      base_time = GST_ELEMENT_CAST (pipeline)->base_time;
+      clock = GST_ELEMENT_CLOCK (pipeline);
+
+      if (clock) {
+        clock_time = gst_clock_get_time (clock);
+        running_time = clock_time - base_time;
+      }
+    } else {
+      running_time = GST_ELEMENT_START_TIME (pipeline);
+    }
+
+    if (!GST_CLOCK_TIME_IS_VALID (running_time)) {
+      GST_OBJECT_UNLOCK (pipeline);
+      return FALSE;
+    }
+
+    if (GST_CLOCK_TIME_IS_VALID (pipeline->priv->instant_rate_upstream_anchor)) {
+      /* Already had an override, calculate the adjustment due to that
+       * elapsed duration */
+      GstClockTime elapsed =
+          running_time - pipeline->priv->instant_rate_clock_anchor;
+      pipeline->priv->instant_rate_upstream_anchor +=
+          elapsed * pipeline->priv->active_instant_rate;
+      pipeline->priv->instant_rate_clock_anchor = running_time;
+    } else {
+      /* Else this is the first override event */
+      pipeline->priv->instant_rate_upstream_anchor =
+          pipeline->priv->instant_rate_clock_anchor = running_time;
+    }
+    upstream_running_time = pipeline->priv->instant_rate_upstream_anchor;
+
+    pipeline->priv->instant_rate_seqnum = seqnum;
+    pipeline->priv->active_instant_rate = rate;
+  }
+
+  GST_OBJECT_UNLOCK (pipeline);
+
+  GST_DEBUG_OBJECT (pipeline,
+      "Instant rate multiplier to %f rt %" GST_TIME_FORMAT " upstream %"
+      GST_TIME_FORMAT, rate, GST_TIME_ARGS (running_time),
+      GST_TIME_ARGS (upstream_running_time));
+
+  event =
+      gst_event_new_instant_rate_sync_time (rate, running_time,
+      upstream_running_time);
+  gst_event_set_seqnum (event, seqnum);
+
+  return gst_element_send_event (GST_ELEMENT_CAST (pipeline), event);
+}
+
+/**
+ * gst_pipeline_is_live:
+ * @pipeline: a #GstPipeline
+ *
+ * Check if @pipeline is live.
+ *
+ * Returns: %TRUE if @pipeline is live, %FALSE if not or if it did not reach the PAUSED state yet.
+ *
+ * MT safe.
+ *
+ * Since: 1.24
+ */
+gboolean
+gst_pipeline_is_live (GstPipeline * pipeline)
+{
+  gboolean is_live;
+
+  GST_OBJECT_LOCK (pipeline);
+  is_live = pipeline->priv->is_live;
+  GST_OBJECT_UNLOCK (pipeline);
+
+  return is_live;
+}
+
+/**
+ * gst_pipeline_get_configured_latency:
+ * @pipeline: a #GstPipeline
+ *
+ * Return the configured latency on @pipeline.
+ *
+ * Returns: @pipeline configured latency, or %GST_CLOCK_TIME_NONE if none has been configured
+ * because @pipeline did not reach the PLAYING state yet.
+ *
+ * MT safe.
+ *
+ * Since: 1.24
+ */
+GstClockTime
+gst_pipeline_get_configured_latency (GstPipeline * pipeline)
+{
+  GstClockTime min_latency;
+
+  GST_OBJECT_LOCK (pipeline);
+  min_latency = pipeline->priv->min_latency;
+  GST_OBJECT_UNLOCK (pipeline);
+
+  return min_latency;
+}
