@@ -60,6 +60,129 @@
 #include <mach/mach_time.h>
 #endif
 
+#if defined __APPLE__
+static struct mach_timebase_info mach_timebase;
+#endif
+
+#if defined G_OS_WIN32
+static LARGE_INTEGER performance_counter_frequency;
+#endif
+
+/* Small helper to make the atomics below cheaper.
+ *
+ * GLib always uses SEQ_CST atomic ops while here it's more than enough to use
+ * ACQUIRE/RELEASE atomic ops. On x86 / x86-64 the ACQUIRE load is compiling
+ * to a simple memory read.
+ */
+#if defined __APPLE__ || defined G_OS_WIN32
+#if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L && !defined(__STDC_NO_ATOMICS)
+#include <stdatomic.h>
+
+typedef atomic_int gst_atomic_int;
+
+static inline int
+gst_atomic_int_get_acquire (gst_atomic_int * x)
+{
+  return atomic_load_explicit (x, memory_order_acquire);
+}
+
+static inline void
+gst_atomic_int_set_release (gst_atomic_int * x, gint val)
+{
+  atomic_store_explicit (x, val, memory_order_release);
+}
+#elif defined G_OS_WIN32
+/* MSVC's C11 atomic might require special cflags
+ * https://devblogs.microsoft.com/cppblog/c11-atomics-in-visual-studio-2022-version-17-5-preview-2/
+ *
+ * Can remove this code once below GLib MR is merged
+ * https://gitlab.gnome.org/GNOME/glib/-/merge_requests/3436
+ */
+
+typedef LONG gst_atomic_int;
+
+static inline int
+gst_atomic_int_get_acquire (gst_atomic_int * x)
+{
+  return InterlockedAndAcquire (x, 1);
+}
+
+static inline void
+gst_atomic_int_set_release (gst_atomic_int * x, gint val)
+{
+  InterlockedOrRelease (x, 1);
+}
+#else /* defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L && !defined(__STDC_NO_ATOMICS) */
+typedef int gst_atomic_int;
+#define gst_atomic_int_get_acquire(x) g_atomic_int_get(x)
+#define gst_atomic_int_set_release(x, val) g_atomic_int_set(x, val)
+#endif /* defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L && !defined(__STDC_NO_ATOMICS) */
+#endif /* defined __APPLE__ || defined G_OS_WIN32 */
+
+/* priv_gst_clock_init:
+ *
+ * Initialize internal state of the clock. This is safe to call multiple
+ * times.
+ */
+void
+priv_gst_clock_init (void)
+{
+#if defined __APPLE__
+  static gst_atomic_int inited = FALSE;
+
+  if (!gst_atomic_int_get_acquire (&inited)) {
+    mach_timebase_info (&mach_timebase);
+    gst_atomic_int_set_release (&inited, TRUE);
+  }
+#endif
+
+#if defined G_OS_WIN32
+  static gst_atomic_int inited = FALSE;
+
+  if (!gst_atomic_int_get_acquire (&inited)) {
+    QueryPerformanceFrequency (&performance_counter_frequency);
+    gst_atomic_int_set_release (&inited, TRUE);
+  }
+#endif
+}
+
+GstClockTime
+priv_gst_get_monotonic_time (void)
+{
+#if defined __APPLE__
+  guint64 mach_t = mach_absolute_time ();
+  return gst_util_uint64_scale (mach_t, mach_timebase.numer,
+      mach_timebase.denom);
+#elif defined G_OS_WIN32
+  LARGE_INTEGER now;
+  QueryPerformanceCounter (&now);
+
+  return gst_util_uint64_scale (now.QuadPart, GST_SECOND,
+      performance_counter_frequency.QuadPart);
+#elif defined (HAVE_POSIX_TIMERS) && defined(HAVE_MONOTONIC_CLOCK) &&\
+    defined (HAVE_CLOCK_GETTIME)
+  struct timespec now;
+
+  clock_gettime (CLOCK_MONOTONIC, &now);
+  return GST_TIMESPEC_TO_TIME (now);
+#else
+  return g_get_monotonic_time () * 1000;
+#endif
+}
+
+GstClockTime
+priv_gst_get_real_time (void)
+{
+#if defined (HAVE_POSIX_TIMERS) && defined (HAVE_CLOCK_GETTIME)
+  struct timespec now;
+
+  clock_gettime (CLOCK_REALTIME, &now);
+  return GST_TIMESPEC_TO_TIME (now);
+#else
+  return g_get_real_time () * 1000;
+#endif
+}
+
 /* Define this to get some extra debug about jitter from each clock_wait */
 #undef WAIT_DEBUGGING
 
@@ -70,11 +193,15 @@
 #define GST_SYSTEM_CLOCK_WAIT(clock)            g_cond_wait(GST_SYSTEM_CLOCK_GET_COND(clock),GST_SYSTEM_CLOCK_GET_LOCK(clock))
 #define GST_SYSTEM_CLOCK_BROADCAST(clock)       g_cond_broadcast(GST_SYSTEM_CLOCK_GET_COND(clock))
 
-#if defined(HAVE_FUTEX)
+#if defined(HAVE_FUTEX) || defined(HAVE_FUTEX_TIME64)
 #include <unistd.h>
 #include <linux/futex.h>
 #include <sys/syscall.h>
 
+#if !defined(__NR_futex) && !defined(__NR_futex_time64)
+#error "Neither __NR_futex nor __NR_futex_time64 are defined but were found by meson"
+#endif
+
 #ifndef FUTEX_WAIT_BITSET_PRIVATE
 #define FUTEX_WAIT_BITSET_PRIVATE FUTEX_WAIT_BITSET
 #endif
@@ -123,14 +250,39 @@ gst_futex_cond_broadcast (guint * cond_val)
 {
   g_atomic_int_inc (cond_val);
 
+#if defined(HAVE_FUTEX_TIME64)
+#if defined(__BIONIC__)
+  if (__builtin_available (android 30, *)) {
+#else
+  {
+#endif
+    int res;
+    res = syscall (__NR_futex_time64, cond_val, (gsize) FUTEX_WAKE_PRIVATE,
+        (gsize) INT_MAX, NULL);
+
+    /* If the syscall does not exist (`ENOSYS`), we retry again below with the
+     * normal `futex` syscall. This can happen if newer kernel headers are
+     * used than the kernel that is actually running.
+     */
+#if defined(HAVE_FUTEX)
+    if (res >= 0 || errno != ENOSYS) {
+#else
+    {
+#endif
+      return;
+    }
+  }
+#endif
+
+#if defined(HAVE_FUTEX)
   syscall (__NR_futex, cond_val, (gsize) FUTEX_WAKE_PRIVATE, (gsize) INT_MAX,
       NULL);
+#endif
 }
 
 static gboolean
 gst_futex_cond_wait_until (guint * cond_val, GMutex * mutex, gint64 end_time)
 {
-  struct timespec end;
   guint sampled;
   int res;
   gboolean success;
@@ -138,20 +290,96 @@ gst_futex_cond_wait_until (guint * cond_val, GMutex * mutex, gint64 end_time)
   if (end_time < 0)
     return FALSE;
 
-  end.tv_sec = end_time / 1000000000;
-  end.tv_nsec = end_time % 1000000000;
-
   sampled = *cond_val;
   g_mutex_unlock (mutex);
-  /* we use FUTEX_WAIT_BITSET_PRIVATE rather than FUTEX_WAIT_PRIVATE to be
-   * able to use absolute time */
-  res =
-      syscall (__NR_futex, cond_val, (gsize) FUTEX_WAIT_BITSET_PRIVATE,
-      (gsize) sampled, &end, NULL, FUTEX_BITSET_MATCH_ANY);
-  success = (res < 0 && errno == ETIMEDOUT) ? FALSE : TRUE;
-  g_mutex_lock (mutex);
 
-  return success;
+  /* `struct timespec` as defined by the libc headers does not necessarily
+   * have any relation to the one used by the kernel for the `futex` syscall.
+   *
+   * Specifically, the libc headers might use 64-bit `time_t` while the kernel
+   * headers use 32-bit `__kernel_old_time_t` on certain systems.
+   *
+   * To get around this problem we
+   *   a) check if `futex_time64` is available, which only exists on 32-bit
+   *      platforms and always uses 64-bit `time_t`.
+   *   b) otherwise (or if that returns `ENOSYS`), we call the normal `futex`
+   *      syscall with the `struct timespec_t` used by the kernel, which uses
+   *      `__kernel_long_t` for both its fields. We use that instead of
+   *      `__kernel_old_time_t` because it is equivalent and available in the
+   *      kernel headers for a longer time.
+   *
+   * Also some 32-bit systems do not define `__NR_futex` at all and only
+   * define `__NR_futex_time64`.
+   */
+
+#if defined(HAVE_FUTEX_TIME64)
+#if defined(__BIONIC__)
+  if (__builtin_available (android 30, *)) {
+#else
+  {
+#endif
+    struct
+    {
+      gint64 tv_sec;
+      gint64 tv_nsec;
+    } end;
+
+    end.tv_sec = end_time / 1000000000;
+    end.tv_nsec = end_time % 1000000000;
+
+    /* we use FUTEX_WAIT_BITSET_PRIVATE rather than FUTEX_WAIT_PRIVATE to be
+     * able to use absolute time */
+    res =
+        syscall (__NR_futex_time64, cond_val, (gsize) FUTEX_WAIT_BITSET_PRIVATE,
+        (gsize) sampled, &end, NULL, FUTEX_BITSET_MATCH_ANY);
+
+    /* If the syscall does not exist (`ENOSYS`), we retry again below with the
+     * normal `futex` syscall. This can happen if newer kernel headers are
+     * used than the kernel that is actually running.
+     */
+#if defined(HAVE_FUTEX)
+    if (res >= 0 || errno != ENOSYS) {
+#else
+    {
+#endif
+      success = (res < 0 && errno == ETIMEDOUT) ? FALSE : TRUE;
+      g_mutex_lock (mutex);
+
+      return success;
+    }
+  }
+#endif
+
+#if defined(HAVE_FUTEX)
+  {
+    struct
+    {
+      __kernel_long_t tv_sec;
+      __kernel_long_t tv_nsec;
+    } end;
+
+    /* Make sure to only ever call this if the end time actually fits into the
+     * target type */
+    g_assert (sizeof (__kernel_long_t) >= 8
+        || end_time / 1000000000 <= G_MAXINT32);
+
+    end.tv_sec = end_time / 1000000000;
+    end.tv_nsec = end_time % 1000000000;
+
+    /* we use FUTEX_WAIT_BITSET_PRIVATE rather than FUTEX_WAIT_PRIVATE to be
+     * able to use absolute time */
+    res =
+        syscall (__NR_futex, cond_val, (gsize) FUTEX_WAIT_BITSET_PRIVATE,
+        (gsize) sampled, &end, NULL, FUTEX_BITSET_MATCH_ANY);
+    success = (res < 0 && errno == ETIMEDOUT) ? FALSE : TRUE;
+    g_mutex_lock (mutex);
+
+    return success;
+  }
+#endif
+
+  /* We can't end up here because of the checks above */
+  g_assert_not_reached ();
 }
 
 #elif defined (G_OS_UNIX)
@@ -203,7 +431,7 @@ gst_pthread_cond_wait_until (pthread_cond_t * cond, pthread_mutex_t * lock,
    * Since this pthreads wants the relative time, convert it back again.
    */
   {
-    gint64 now = g_get_monotonic_time () * 1000;
+    gint64 now = priv_gst_get_monotonic_time ();
     gint64 relative;
 
     if (end_time <= now)
@@ -349,13 +577,6 @@ struct _GstSystemClockPrivate
   GCond entries_changed;
 
   GstClockType clock_type;
-
-#ifdef G_OS_WIN32
-  LARGE_INTEGER frequency;
-#endif                          /* G_OS_WIN32 */
-#ifdef __APPLE__
-  struct mach_timebase_info mach_timebase;
-#endif
 };
 
 #ifdef HAVE_POSIX_TIMERS
@@ -365,7 +586,7 @@ struct _GstSystemClockPrivate
 #  define DEFAULT_CLOCK_TYPE GST_CLOCK_TYPE_REALTIME
 # endif
 #else
-#define DEFAULT_CLOCK_TYPE GST_CLOCK_TYPE_REALTIME
+#define DEFAULT_CLOCK_TYPE GST_CLOCK_TYPE_MONOTONIC
 #endif
 
 enum
@@ -450,14 +671,6 @@ gst_system_clock_init (GstSystemClock * clock)
   priv->entries = NULL;
   g_cond_init (&priv->entries_changed);
 
-#ifdef G_OS_WIN32
-  QueryPerformanceFrequency (&priv->frequency);
-#endif /* G_OS_WIN32 */
-
-#ifdef __APPLE__
-  mach_timebase_info (&priv->mach_timebase);
-#endif
-
 #if 0
   /* Uncomment this to start the async clock thread straight away */
   GST_SYSTEM_CLOCK_LOCK (clock);
@@ -808,7 +1021,6 @@ clock_type_to_posix_id (GstClockType clock_type)
 #ifdef HAVE_MONOTONIC_CLOCK
   if (clock_type == GST_CLOCK_TYPE_MONOTONIC)
     return CLOCK_MONOTONIC;
-  else
 #endif
   if (clock_type == GST_CLOCK_TYPE_TAI)
 #ifdef CLOCK_TAI
@@ -825,68 +1037,59 @@ clock_type_to_posix_id (GstClockType clock_type)
 static GstClockTime
 gst_system_clock_get_internal_time (GstClock * clock)
 {
-#if defined __APPLE__
-  GstSystemClock *sysclock = GST_SYSTEM_CLOCK_CAST (clock);
-  uint64_t mach_t = mach_absolute_time ();
-  return gst_util_uint64_scale (mach_t, sysclock->priv->mach_timebase.numer,
-      sysclock->priv->mach_timebase.denom);
-#else
-#ifdef G_OS_WIN32
   GstSystemClock *sysclock = GST_SYSTEM_CLOCK_CAST (clock);
 
-  if (sysclock->priv->frequency.QuadPart != 0) {
-    LARGE_INTEGER now;
+  /* For the monotonic and realtime clock, always directly call the specific
+   * helper functions above */
+  if (sysclock->priv->clock_type == GST_CLOCK_TYPE_MONOTONIC)
+    return priv_gst_get_monotonic_time ();
+  else if (sysclock->priv->clock_type == GST_CLOCK_TYPE_REALTIME)
+    return priv_gst_get_real_time ();
 
-    /* we prefer the highly accurate performance counters on windows */
-    QueryPerformanceCounter (&now);
+  /* If POSIX timers are available, use those for any other clock,
+   * or otherwise return the monotonic time */
+#if defined HAVE_POSIX_TIMERS && defined HAVE_CLOCK_GETTIME
+  // BSD and Linux' Posix timers and clock_gettime cover all of the different clock types
+  // without need for special handling so we'll use those.
+  clockid_t ptype;
+  struct timespec ts;
 
-    return gst_util_uint64_scale (now.QuadPart,
-        GST_SECOND, sysclock->priv->frequency.QuadPart);
-  } else
-#endif /* G_OS_WIN32 */
-#if !defined HAVE_POSIX_TIMERS || !defined HAVE_CLOCK_GETTIME
-  {
-    gint64 monotime;
+  ptype = clock_type_to_posix_id (sysclock->priv->clock_type);
 
-    monotime = g_get_monotonic_time ();
+  if (G_UNLIKELY (clock_gettime (ptype, &ts)))
+    return GST_CLOCK_TIME_NONE;
 
-    return monotime * 1000;
-  }
-#else
-  {
-    GstSystemClock *sysclock = GST_SYSTEM_CLOCK_CAST (clock);
-    clockid_t ptype;
-    struct timespec ts;
-
-    ptype = clock_type_to_posix_id (sysclock->priv->clock_type);
-
-    if (G_UNLIKELY (clock_gettime (ptype, &ts)))
-      return GST_CLOCK_TIME_NONE;
-
-    return GST_TIMESPEC_TO_TIME (ts);
-  }
-#endif
-#endif /* __APPLE__ */
+  return GST_TIMESPEC_TO_TIME (ts);
+#else /* !HAVE_POSIX_TIMERS || !HAVE_CLOCK_GETTIME */
+  return priv_gst_get_monotonic_time ();
+#endif /* !HAVE_POSIX_TIMERS || !HAVE_CLOCK_GETTIME */
 }
 
 static guint64
 gst_system_clock_get_resolution (GstClock * clock)
 {
-#if defined __APPLE__
-  GstSystemClock *sysclock = GST_SYSTEM_CLOCK_CAST (clock);
-  return gst_util_uint64_scale (GST_NSECOND,
-      sysclock->priv->mach_timebase.numer, sysclock->priv->mach_timebase.denom);
-#else
-#ifdef G_OS_WIN32
   GstSystemClock *sysclock = GST_SYSTEM_CLOCK_CAST (clock);
 
-  if (sysclock->priv->frequency.QuadPart != 0) {
-    return GST_SECOND / sysclock->priv->frequency.QuadPart;
-  } else
-#endif /* G_OS_WIN32 */
+#if defined G_OS_WIN32
+  if (sysclock->priv->clock_type == GST_CLOCK_TYPE_REALTIME) {
+    return 1 * GST_USECOND;
+  } else {
+    return GST_SECOND / performance_counter_frequency.QuadPart;
+  }
+#endif
+
+#if defined __APPLE__
+  // On Apple platforms we want to use mach_absolute_time() unconditionally
+  // for the monotonic clock even if clock_gettime() support is available.
+  // Only use the latter for other clock types there.
+  if (sysclock->priv->clock_type == GST_CLOCK_TYPE_MONOTONIC) {
+    return gst_util_uint64_scale (GST_NSECOND,
+        mach_timebase.numer, mach_timebase.denom);
+  }
+#endif
+
 #if defined(HAVE_POSIX_TIMERS) && defined(HAVE_CLOCK_GETTIME)
   {
-    GstSystemClock *sysclock = GST_SYSTEM_CLOCK_CAST (clock);
     clockid_t ptype;
     struct timespec ts;
 
@@ -897,12 +1100,9 @@ gst_system_clock_get_resolution (GstClock * clock)
 
     return GST_TIMESPEC_TO_TIME (ts);
   }
-#else
-  {
-    return 1 * GST_USECOND;
-  }
-#endif
-#endif /* __APPLE__ */
+#endif /* HAVE_POSIX_TIMERS && HAVE_CLOCK_GETTIME */
+
+  return 1 * GST_USECOND;
 }
 
 /* synchronously wait on the given GstClockEntry.
@@ -961,11 +1161,34 @@ gst_system_clock_id_wait_jitter_unlocked (GstClock * clock,
     while (TRUE) {
       gboolean waitret;
 
-      /* now wait on the entry, it either times out or the cond is signalled.
-       * The status of the entry is BUSY only around the wait. */
-      waitret =
-          GST_SYSTEM_CLOCK_ENTRY_WAIT_UNTIL ((GstClockEntryImpl *) entry,
-          mono_ts * 1000 + diff);
+#ifdef HAVE_CLOCK_NANOSLEEP
+      if (diff <= 500 * GST_USECOND) {
+        /* In order to provide more accurate wait, we will use BLOCKING
+           clock_nanosleep for any deadlines at or below 500us */
+        struct timespec end;
+        GST_TIME_TO_TIMESPEC (mono_ts * 1000 + diff, end);
+        GST_SYSTEM_CLOCK_ENTRY_UNLOCK ((GstClockEntryImpl *) entry);
+        waitret =
+            clock_nanosleep (CLOCK_MONOTONIC, TIMER_ABSTIME, &end, NULL) == 0;
+        GST_SYSTEM_CLOCK_ENTRY_LOCK ((GstClockEntryImpl *) entry);
+      } else {
+
+        if (diff < 2 * GST_MSECOND) {
+          /* For any deadline within 2ms, we first use the regular non-blocking
+             wait by reducing the diff accordingly */
+          diff -= 500 * GST_USECOND;
+        }
+#endif
+
+        /* now wait on the entry, it either times out or the cond is signalled.
+         * The status of the entry is BUSY only around the wait. */
+        waitret =
+            GST_SYSTEM_CLOCK_ENTRY_WAIT_UNTIL ((GstClockEntryImpl *) entry,
+            mono_ts * 1000 + diff);
+
+#ifdef HAVE_CLOCK_NANOSLEEP
+      }
+#endif
 
       /* get the new status, mark as DONE. We do this so that the unschedule
        * function knows when we left the poll and doesn't need to wakeup the
@@ -1004,6 +1227,7 @@ gst_system_clock_id_wait_jitter_unlocked (GstClock * clock,
 
         /* reschedule if gst_cond_wait_until returned early or we have to reschedule after
          * an unlock*/
+        mono_ts = g_get_monotonic_time ();
         now = gst_clock_get_time (clock);
         diff = GST_CLOCK_DIFF (now, entryt);
 
