@@ -137,12 +137,15 @@ struct _GstNetClientInternalClock
   GSocketAddress *servaddr;
   GCancellable *cancel;
   gboolean made_cancel_fd;
+  gboolean marked_corrupted;
 
   GstClockTime timeout_expiration;
   GstClockTime roundtrip_limit;
   GstClockTime rtt_avg;
   GstClockTime minimum_update_interval;
   GstClockTime last_remote_poll_interval;
+  GstClockTime last_remote_time;
+  GstClockTime remote_avg_old;
   guint skipped_updates;
   GstClockTime last_rtts[MEDIAN_PRE_FILTERING_WINDOW];
   gint last_rtts_missing;
@@ -229,6 +232,8 @@ gst_net_client_internal_clock_init (GstNetClientInternalClock * self)
   self->last_remote_poll_interval = GST_CLOCK_TIME_NONE;
   self->skipped_updates = 0;
   self->last_rtts_missing = MEDIAN_PRE_FILTERING_WINDOW;
+  self->marked_corrupted = FALSE;
+  self->remote_avg_old = 0;
 }
 
 static void
@@ -372,6 +377,11 @@ gst_net_client_internal_clock_observe_times (GstNetClientInternalClock * self,
       G_GUINT64_FORMAT " local2 %" G_GUINT64_FORMAT, local_1, remote_1,
       remote_2, local_2);
 
+  /* if we are ahead of time the time server may have been reset */
+  if (self->last_remote_time > remote_1 || self->marked_corrupted)
+    goto corrupted;
+  self->last_remote_time = remote_1;
+
   /* If the server told us a poll interval and it's bigger than the
    * one configured via the property, use the server's */
   if (self->last_remote_poll_interval != GST_CLOCK_TIME_NONE &&
@@ -519,6 +529,20 @@ gst_net_client_internal_clock_observe_times (GstNetClientInternalClock * self,
       && GST_CLOCK_DIFF (time_before,
           remote_avg) < (GstClockTimeDiff) (max_discont));
 
+  /* Check if new remote_avg is less than before to detect if signal lost
+   * sync due to the remote clock has restarted. Then the new remote time will
+   * be less than the previous time which should not happen if increased in a
+   * monotonic way. Also, only perform this check on a synchronized clock to
+   * avoid startup issues.
+   */
+  if (synched) {
+    if (remote_avg < self->remote_avg_old) {
+      gst_clock_set_synced (GST_CLOCK (self), FALSE);
+    } else {
+      self->remote_avg_old = remote_avg;
+    }
+  }
+
   if (gst_clock_add_observation_unapplied (GST_CLOCK_CAST (self),
           local_avg, remote_avg, &r_squared, &internal_time, &external_time,
           &rate_num, &rate_den)) {
@@ -633,6 +657,14 @@ bogus_observation:
   /* Schedule a new packet again soon */
   self->timeout_expiration = gst_util_get_timestamp () + (GST_SECOND / 4);
   return;
+
+corrupted:
+  if (!self->marked_corrupted) {
+    GST_ERROR_OBJECT (self, "Remote clock time reverted, mark clock invalid");
+    self->marked_corrupted = TRUE;
+  }
+  GST_OBJECT_UNLOCK (self);
+  return;
 }
 
 static gpointer
@@ -677,7 +709,7 @@ gst_net_client_internal_clock_thread (gpointer data)
         /* before next sending check if need to change QoS */
         new_qos_dscp = self->qos_dscp;
         if (cur_qos_dscp != new_qos_dscp &&
-            gst_net_utils_set_socket_dscp (socket, new_qos_dscp)) {
+            gst_net_utils_set_socket_tos (socket, new_qos_dscp)) {
           GST_DEBUG_OBJECT (self, "changed QoS DSCP to: %d", new_qos_dscp);
           cur_qos_dscp = new_qos_dscp;
         }
@@ -1180,7 +1212,13 @@ gst_net_client_clock_finalize (GObject * object)
         update_clock_cache (cache);
       } else {
         GstClock *sysclock = gst_system_clock_obtain ();
-        GstClockTime time = gst_clock_get_time (sysclock) + 60 * GST_SECOND;
+        GstClockTime time = gst_clock_get_time (sysclock);
+        GstNetClientInternalClock *internal_clock =
+            GST_NET_CLIENT_INTERNAL_CLOCK (cache->clock);
+
+        /* only defer deletion if the clock is not marked corrupted */
+        if (!internal_clock->marked_corrupted)
+          time += 60 * GST_SECOND;
 
         cache->remove_id = gst_clock_new_single_shot_id (sysclock, time);
         gst_clock_id_wait_async (cache->remove_id, remove_clock_cache, cache,
@@ -1349,6 +1387,9 @@ gst_net_client_clock_constructed (GObject * object)
     GstNetClientInternalClock *internal_clock =
         GST_NET_CLIENT_INTERNAL_CLOCK (tmp->clock);
 
+    if (internal_clock->marked_corrupted)
+      break;
+
     if (strcmp (internal_clock->address, self->priv->address) == 0 &&
         internal_clock->port == self->priv->port) {
       cache = tmp;
@@ -1410,12 +1451,12 @@ gst_net_client_clock_get_internal_time (GstClock * clock)
 
 /**
  * gst_net_client_clock_new:
- * @name: a name for the clock
+ * @name: (nullable): a name for the clock
  * @remote_address: the address or hostname of the remote clock provider
  * @remote_port: the port of the remote clock provider
  * @base_time: initial time of the clock
  *
- * Create a new #GstNetClientInternalClock that will report the time
+ * Create a new #GstNetClientClock that will report the time
  * provided by the #GstNetTimeProvider on @remote_address and
  * @remote_port.
  *
@@ -1458,7 +1499,7 @@ gst_ntp_clock_init (GstNtpClock * self)
 
 /**
  * gst_ntp_clock_new:
- * @name: a name for the clock
+ * @name: (nullable): a name for the clock
  * @remote_address: the address or hostname of the remote clock provider
  * @remote_port: the port of the remote clock provider
  * @base_time: initial time of the clock
