@@ -86,6 +86,7 @@ struct _GstQueue2
 
   /* Position of src/sink */
   GstClockTime sinktime, srctime;
+  GstClockTime sink_start_time;
   /* TRUE if either position needs to be recalculated */
   gboolean sink_tainted, src_tainted;
   /* Bitrates taken from tags */
@@ -164,13 +165,12 @@ struct _GstQueue2
   guint64 ring_buffer_max_size;
   guint8 * ring_buffer;
 
-  volatile gint downstream_may_block;
+  gint downstream_may_block;
 
   GstBufferingMode mode;
   gint64 buffering_left;
   gint avg_in;
   gint avg_out;
-  gboolean percent_changed;
   GMutex buffering_post_lock; /* assures only one posted at a time */
 };
 
