@@ -76,7 +76,7 @@ GST_START_TEST (test_num_buffers)
     g_object_set (sinks[i], "signal-handoffs", TRUE, NULL);
     g_signal_connect (sinks[i], "handoff", (GCallback) handoff, &counts[i]);
 
-    req_pads[i] = gst_element_get_request_pad (tee, "src_%u");
+    req_pads[i] = gst_element_request_pad_simple (tee, "src_%u");
     fail_unless (req_pads[i] != NULL);
 
     qpad = gst_element_get_static_pad (queues[i], "sink");
@@ -142,7 +142,7 @@ GST_START_TEST (test_stress)
   for (i = 0; i < 50000; i++) {
     GstPad *pad;
 
-    pad = gst_element_get_request_pad (tee, "src_%u");
+    pad = gst_element_request_pad_simple (tee, "src_%u");
     gst_element_release_request_pad (tee, pad);
     gst_object_unref (pad);
 
@@ -201,7 +201,7 @@ buffer_alloc_harness_setup (BufferAllocHarness * h, gint countdown)
   h->tee_sinkpad = gst_element_get_static_pad (h->tee, "sink");
   fail_if (h->tee_sinkpad == NULL);
 
-  h->tee_srcpad = gst_element_get_request_pad (h->tee, "src_%u");
+  h->tee_srcpad = gst_element_request_pad_simple (h->tee, "src_%u");
   fail_if (h->tee_srcpad == NULL);
 
   h->final_sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
@@ -339,7 +339,8 @@ GST_START_TEST (test_internal_links)
   GstIteratorResult res;
   GValue val1 = { 0, }
   , val2 = {
-  0,};
+    0,
+  };
 
   tee = gst_check_setup_element ("tee");
 
@@ -353,7 +354,7 @@ GST_START_TEST (test_internal_links)
   fail_unless (res == GST_ITERATOR_DONE);
   fail_unless (g_value_get_object (&val1) == NULL);
 
-  srcpad1 = gst_element_get_request_pad (tee, "src_%u");
+  srcpad1 = gst_element_request_pad_simple (tee, "src_%u");
   fail_unless (srcpad1 != NULL);
 
   /* iterator should resync */
@@ -373,7 +374,7 @@ GST_START_TEST (test_internal_links)
   fail_unless (res == GST_ITERATOR_DONE);
   fail_unless (g_value_get_object (&val1) == NULL);
 
-  srcpad2 = gst_element_get_request_pad (tee, "src_%u");
+  srcpad2 = gst_element_request_pad_simple (tee, "src_%u");
   fail_unless (srcpad2 != NULL);
 
   /* iterator should resync */
@@ -467,9 +468,9 @@ GST_START_TEST (test_flow_aggregation)
   fail_unless (tee != NULL);
   teesink = gst_element_get_static_pad (tee, "sink");
   fail_unless (teesink != NULL);
-  teesrc1 = gst_element_get_request_pad (tee, "src_%u");
+  teesrc1 = gst_element_request_pad_simple (tee, "src_%u");
   fail_unless (teesrc1 != NULL);
-  teesrc2 = gst_element_get_request_pad (tee, "src_%u");
+  teesrc2 = gst_element_request_pad_simple (tee, "src_%u");
   fail_unless (teesrc2 != NULL);
 
   GST_DEBUG ("Creating mysink1");
@@ -602,16 +603,16 @@ GST_START_TEST (test_request_pads)
 
   tee = gst_check_setup_element ("tee");
 
-  srcpad1 = gst_element_get_request_pad (tee, "src_%u");
+  srcpad1 = gst_element_request_pad_simple (tee, "src_%u");
   fail_unless (srcpad1 != NULL);
   fail_unless_equals_string (GST_OBJECT_NAME (srcpad1), "src_0");
-  srcpad2 = gst_element_get_request_pad (tee, "src_100");
+  srcpad2 = gst_element_request_pad_simple (tee, "src_100");
   fail_unless (srcpad2 != NULL);
   fail_unless_equals_string (GST_OBJECT_NAME (srcpad2), "src_100");
-  srcpad3 = gst_element_get_request_pad (tee, "src_10");
+  srcpad3 = gst_element_request_pad_simple (tee, "src_10");
   fail_unless (srcpad3 != NULL);
   fail_unless_equals_string (GST_OBJECT_NAME (srcpad3), "src_10");
-  srcpad4 = gst_element_get_request_pad (tee, "src_%u");
+  srcpad4 = gst_element_request_pad_simple (tee, "src_%u");
   fail_unless (srcpad4 != NULL);
 
   gst_object_unref (srcpad1);
@@ -661,11 +662,11 @@ GST_START_TEST (test_allow_not_linked)
 
   fail_unless (gst_pad_push (srcpad, gst_buffer_ref (buffer)) == GST_FLOW_OK);
 
-  src1 = gst_element_get_request_pad (tee, "src_%u");
+  src1 = gst_element_request_pad_simple (tee, "src_%u");
 
   fail_unless (gst_pad_push (srcpad, gst_buffer_ref (buffer)) == GST_FLOW_OK);
 
-  src2 = gst_element_get_request_pad (tee, "src_%u");
+  src2 = gst_element_request_pad_simple (tee, "src_%u");
 
   fail_unless (gst_pad_push (srcpad, gst_buffer_ref (buffer)) == GST_FLOW_OK);
 
@@ -850,7 +851,7 @@ GST_START_TEST (test_allocation_query_allow_not_linked)
   add_sink_pad_and_setup_query_func (tee, allocation_query3);
   /* This unlinked pad is what will make a difference between having
    * allow-not-linked set or not */
-  srcpad = gst_element_get_request_pad (tee, "src_%u");
+  srcpad = gst_element_request_pad_simple (tee, "src_%u");
   caps = gst_caps_new_empty_simple ("test/test");
 
   /* Without allow-not-linked the query should fail */
