@@ -83,6 +83,56 @@ GST_START_TEST (test_add_pad_unref_element)
 
 GST_END_TEST;
 
+static void
+test_add_pad_while_paused_dummy_task (void *user_data)
+{
+  GstPad *pad = (GstPad *) user_data;
+  gst_pad_pause_task (pad);
+}
+
+static gboolean
+test_add_pad_while_paused_pad_activatemode (GstPad * pad, GstObject * parent,
+    GstPadMode mode, gboolean active)
+{
+  *(gboolean *) pad->activatemodedata = active;
+  fail_unless (mode == GST_PAD_MODE_PUSH);
+  if (active)
+    gst_pad_start_task (pad, test_add_pad_while_paused_dummy_task, pad, NULL);
+  else
+    gst_pad_stop_task (pad);
+  return TRUE;
+}
+
+GST_START_TEST (test_add_pad_while_paused)
+{
+  GstElement *e;
+  GstPad *p;
+  gboolean active = FALSE;
+
+  e = gst_element_factory_make ("fakesrc", "source");
+  gst_element_set_state (e, GST_STATE_PAUSED);
+  {
+    GstPad *old_pad = gst_element_get_static_pad (e, "src");
+    gst_pad_set_active (old_pad, FALSE);
+    gst_element_remove_pad (e, old_pad);
+    gst_object_unref (old_pad);
+  }
+
+  p = gst_pad_new ("dynamic", GST_PAD_SRC);
+  gst_pad_set_activatemode_function_full (p,
+      test_add_pad_while_paused_pad_activatemode, (void *) &active, NULL);
+
+  fail_if (active);
+  gst_element_add_pad (e, p);
+  fail_if (!active);
+  gst_element_set_state (e, GST_STATE_NULL);
+  fail_if (active);
+
+  gst_object_unref (e);
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_error_no_bus)
 {
   GstElement *e;
@@ -654,7 +704,6 @@ GST_START_TEST (test_request_pad_templates)
   GHashTableIter iter;
   gpointer key, value;
   const gchar *pad_name, *templ_name;
-  GSList *padname_blacklists = NULL, *item;
   GError *err = NULL;
 
   padnames = g_hash_table_new (g_str_hash, g_str_equal);
@@ -703,23 +752,7 @@ GST_START_TEST (test_request_pad_templates)
   g_hash_table_insert (padnames, (gpointer) "src_%s", (gpointer) "src_%s");
   g_hash_table_insert (padnames, (gpointer) "src_%u_%s",
       (gpointer) "src_%u_%s");
-
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u%u");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u_%d");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u_%u_");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%u_%s_%s");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_%u");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s_%s");
-  padname_blacklists =
-      g_slist_append (padname_blacklists, (gpointer) "src_%s_blah");
+  g_hash_table_insert (padnames, (gpointer) "src_foo_bar", (gpointer) "src_%s");
 
   test = g_object_new (gst_test_element3_get_type (), NULL);
 
@@ -742,16 +775,6 @@ GST_START_TEST (test_request_pad_templates)
     gst_object_unref (pad);
   }
 
-  item = padname_blacklists;
-
-  /* check invalid request pad name */
-  while (item) {
-    pad_name = (const gchar *) (item->data);
-    item = g_slist_next (item);
-    pad = gst_element_request_pad_simple (GST_ELEMENT (test), pad_name);
-    fail_unless (pad == NULL);
-  }
-
   /* check it working with some APIs
    * gst_element_link/link_pads */
   sink = gst_element_factory_make ("fakesink", "sink");
@@ -782,13 +805,45 @@ GST_START_TEST (test_request_pad_templates)
   if (err) {
     g_error_free (err);
   }
-  g_slist_free (padname_blacklists);
   g_hash_table_unref (padnames);
   gst_object_unref (pipeline);
 }
 
 GST_END_TEST;
 
+GST_START_TEST (test_forbidden_pad_template_names)
+{
+  const gchar *pad_name;
+  GSList *padname_blacklists = NULL, *item;
+
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%u%u");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%u_%s_%s");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_%u");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_%s_%s");
+  padname_blacklists =
+      g_slist_append (padname_blacklists, (gpointer) "src_%s_blah");
+
+  item = padname_blacklists;
+
+  /* check invalid request pad name */
+  while (item) {
+    pad_name = (const gchar *) (item->data);
+    item = g_slist_next (item);
+    ASSERT_WARNING (gst_pad_template_new (pad_name, GST_PAD_SRC,
+            GST_PAD_REQUEST, GST_CAPS_ANY));
+  }
+
+  g_slist_free (padname_blacklists);
+}
+
+GST_END_TEST;
+
 static gboolean run_foreach_thread;
 
 /* thread function that just adds/removes pads while main thread iterates pads */
@@ -916,12 +971,14 @@ gst_element_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_add_remove_pad);
   tcase_add_test (tc_chain, test_add_pad_unref_element);
+  tcase_add_test (tc_chain, test_add_pad_while_paused);
   tcase_add_test (tc_chain, test_error_no_bus);
   tcase_add_test (tc_chain, test_link);
   tcase_add_test (tc_chain, test_link_no_pads);
   tcase_add_test (tc_chain, test_pad_templates);
   tcase_add_test (tc_chain, test_property_notify_message);
   tcase_add_test (tc_chain, test_request_pad_templates);
+  tcase_add_test (tc_chain, test_forbidden_pad_template_names);
   tcase_add_test (tc_chain, test_foreach_pad);
 
   return s;
