@@ -779,7 +779,7 @@ GST_START_TEST (test_serialize_nested_structures)
       ", main-sub1=(structure)[type-b, machine-type=(int)0;]"
       ", main-sub2=(structure)[type-a, plugin-filename=(string)\"/home/user/lib/lib\\ with\\ spaces.dll\", machine-type=(int)1;]"
       ", main-sub3=(structure)[type-b, plugin-filename=(string)/home/user/lib/lib_no_spaces.so, machine-type=(int)1;]"
-      ";";
+      ", main-sub4=(structure){ [s1, a=(int)1;], [s2, b=(int)2;] }" ";";
 
   s = gst_structure_from_string (str1, &end);
   fail_unless (s != NULL);
@@ -787,15 +787,15 @@ GST_START_TEST (test_serialize_nested_structures)
   GST_DEBUG ("not parsed part : %s", end);
   fail_unless (*end == '\0');
 
-  fail_unless (gst_structure_n_fields (s) == 3);
+  fail_unless (gst_structure_n_fields (s) == 4);
 
   fail_unless (gst_structure_has_field_typed (s, "main-sub1",
           GST_TYPE_STRUCTURE));
 
-  str2 = gst_structure_serialize (s, GST_SERIALIZE_FLAG_NONE);
+  str2 = gst_structure_serialize_full (s, GST_SERIALIZE_FLAG_NONE);
   fail_unless (str2 != NULL);
 
-  fail_unless (g_str_equal (str1, str2));
+  fail_unless_equals_string (str1, str2);
 
   g_free (str2);
 
@@ -1018,6 +1018,41 @@ GST_START_TEST (test_flagset)
 
 GST_END_TEST;
 
+GST_START_TEST (test_flags)
+{
+  GstStructure *s;
+  GstSeekFlags flags = GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT;
+  guint parsed_flags = 0;
+
+  s = gst_structure_new ("test-struct", "test-seek-flags",
+      GST_TYPE_SEEK_FLAGS, flags, NULL);
+  fail_unless (gst_structure_get_flags (s, "test-seek-flags",
+          GST_TYPE_SEEK_FLAGS, &parsed_flags));
+  fail_unless (flags == (GstSeekFlags) parsed_flags);
+  gst_structure_free (s);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_strict)
+{
+  GstStructure *s;
+
+  GstElement *bin = gst_bin_new ("mybin");
+  s = gst_structure_new ("test-struct", "obj", GST_TYPE_BIN, bin, NULL);
+  fail_unless (s);
+  fail_if (gst_structure_serialize_full (s, GST_SERIALIZE_FLAG_STRICT));
+  gst_structure_free (s);
+  gst_object_unref (bin);
+
+  s = gst_structure_new ("test-struct", "ptr", G_TYPE_POINTER, NULL, NULL);
+  fail_unless (s);
+  fail_if (gst_structure_serialize_full (s, GST_SERIALIZE_FLAG_STRICT));
+  gst_structure_free (s);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_structure_suite (void)
 {
@@ -1050,6 +1085,8 @@ gst_structure_suite (void)
   tcase_add_test (tc_chain, test_map_in_place);
   tcase_add_test (tc_chain, test_filter_and_map_in_place);
   tcase_add_test (tc_chain, test_flagset);
+  tcase_add_test (tc_chain, test_flags);
+  tcase_add_test (tc_chain, test_strict);
   return s;
 }
 
